
//////////////////////////////////////////////////////////////////////////
// WS2RSVP.H - RSVP Protocol Specific Annex for Winsock2
//
// This file contains RSVP specific definitions for Winsock2 
// applications. It defines the Control Socket programming interface
// to Intel's implementation of Resource ReSerVation Protocol (RSVP).
//
// These definitions are explained in more detail in the Winsock2
// protocol specific documentation.
//
// Revision: 0.4    March 1997
//
//////////////////////////////////////////////////////////////////////////


#ifndef  _W2RSVP_HDR
#define _W2RSVP_HDR

#include <winsock2.h>

// WinSock 2 Provider ID for RSVP
// {0F1E5156-F07A-11CF-980E-00AA00580A07}
static const GUID RSVP_ProviderId =
{ 0xf1e5156, 0xf07a, 0x11cf, { 0x98, 0xe, 0x0, 0xaa, 0x0, 0x58, 0xa, 0x7 } };

// RSVP protocol specific number
#define IPPROTO_RSVP            0x2e  // Protocol ID 46

// Base value for RSVP WSAIoctl commands
#define _IOC_RSVP               ((IOC_PROTOCOL)|((IPPROTO_RSVP)<<16))

// RSVP WSAIoctl commands
#define SIO_RSVP_REGISTER       _WSAIORW(_IOC_RSVP,1)
#define SIO_RSVP_SENDER         _WSAIORW(_IOC_RSVP,2)
#define SIO_RSVP_RESERVE        _WSAIORW(_IOC_RSVP,3)
#define SIO_RSVP_REL_SENDER     _WSAIORW(_IOC_RSVP,4)
#define SIO_RSVP_REL_RESERVE    _WSAIORW(_IOC_RSVP,5)
#define SIO_RSVP_RETRIEVE       _WSAIORW(_IOC_RSVP,6)
#define SIO_RSVP_QUERY_QOS      _WSAIORW(_IOC_RSVP,7)




////////////////////////////////////////////////////////////////////////
// RSVP QOS SPECIFICATION RELATED DEFINTIONS
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// RSVP_TOKEN_BUCKET_TSPEC defines the traffic characteristics 
// of a data flow. 
//
typedef struct _RSVP_TOKEN_BUCKET_TSPEC
{
    FLOAT   tokenRate;             // In bytes/sec
    FLOAT   bucketSize;            // In bytes
    FLOAT   peakRate;              // In bytes/sec
    LONG    minPoliced;            // In bytes
    LONG    maxPktSize;            // In bytes
} RSVP_TOKEN_BUCKET_TSPEC, *LPRSVP_TOKEN_BUCKET_TSPEC;

typedef const RSVP_TOKEN_BUCKET_TSPEC *LPRSVP_CTOKEN_BUCKET_TSPEC;

////////////////////////////////////////////////////////////////////////
// TSPEC TYPES used in making reservations.
// 
typedef enum {  enum_TokenBucket=1,
                enum_TspecTwo } TspecType;

////////////////////////////////////////////////////////////////////////
// RSVP_TSPEC defines the traffic characteristics of a data flow. 
//
typedef struct _RSVP_TSPEC
{
    TspecType        type;                  //Type of tspec 
    LONG             length;                //Length of total structure
    union
    {
        RSVP_TOKEN_BUCKET_TSPEC tbTspec;    //Token Bucket Tspec                            
    }tspec;
} RSVP_TSPEC, *LPRSVP_TSPEC;

typedef const RSVP_TSPEC *LPCRSVP_TSPEC;

//Sender and Receiver Tspecs
typedef RSVP_TSPEC RSVP_SENDER_TSPEC, *LPRSVP_SENDER_TSPEC;
typedef RSVP_TSPEC RSVP_RECEIVER_TSPEC, *LPRSVP_RECEIVER_TSPEC;

typedef const RSVP_SENDER_TSPEC *LPCRSVP_SENDER_TSPEC;
typedef const RSVP_RECEIVER_TSPEC *LPCRSVP_RECEIVER_TSPEC;

//Length macro for RSVP_TSPEC
#define RSVP_TSPEC_LENGTH(TSPEC_TYPE) \
                (sizeof(TspecType) + sizeof(LONG) + sizeof(TSPEC_TYPE))

////////////////////////////////////////////////////////////////////////
// GS_RSPEC defines the rate characteristics of a data flow in the
// Guaranteed QoS service.
//
typedef struct _GS_RSPEC
{
    FLOAT   rate;            // In bytes/sec
    LONG    slackTerm;       // In microseconds
} GS_RSPEC, *LPGS_RSPEC;

typedef const GS_RSPEC *LPCGS_RSPEC;

////////////////////////////////////////////////////////////////////////
// RSPEC TYPES used in making reservations.
// 
typedef enum {  enum_GuaranteedService=1,
                enum_RspecTwo } RspecType;

////////////////////////////////////////////////////////////////////////
// RSVP_RSPEC defines rate characteristics of a data flow.
typedef struct _RSVP_RSPEC
{
    RspecType        type;                //type of rspec 
    LONG             length;              //length of total structure
    union
    {
        GS_RSPEC gsRspec;                //Token Bucket Tspec                            
    }rspec;
} RSVP_RSPEC, *LPRSVP_RSPEC;

typedef const RSVP_RSPEC *LPCRSVP_RSPEC;

//Sender and Receiver Rspecs
typedef RSVP_RSPEC RSVP_SENDER_RSPEC, *LPRSVP_SENDER_RSPEC;
typedef RSVP_RSPEC RSVP_RECEIVER_RSPEC, *LPRSVP_RECEIVER_RSPEC;

typedef const RSVP_SENDER_RSPEC *LPCRSVP_SENDER_RSPEC;
typedef const RSVP_RECEIVER_RSPEC *LPCRSVP_RECEIVER_RSPEC;

//Length macro for RSVP_RSPEC
#define RSVP_RSPEC_LENGTH(RSPEC_TYPE) \
                (sizeof(RspecType) + sizeof(LONG) + sizeof(RSPEC_TYPE))

////////////////////////////////////////////////////////////////////////
// CL_FLOWSPEC specifies the desired Quality of Service for data flow
// to the Controlled Load QoS Service.
//
typedef struct _CL_FLOWSPEC
{
    RSVP_RECEIVER_TSPEC     Tspec;            //requested TSpec
} CL_FLOWSPEC, *LPCL_FLOWSPEC;

typedef const CL_FLOWSPEC *LPCCL_FLOWSPEC;

////////////////////////////////////////////////////////////////////////
// GS_FLOWSPEC specifies the desired Quality of Service for data flow
// to the Guaranteed QoS Service.
//
typedef struct _GS_FLOWSPEC
{
    RSVP_RECEIVER_TSPEC     Tspec;            //requested Tspec
    RSVP_RECEIVER_RSPEC     Rspec;            //requested Rspec
} GS_FLOWSPEC, *LPGS_FLOWSPEC;

typedef const GS_FLOWSPEC *LPCGS_FLOWSPEC;


////////////////////////////////////////////////////////////////////////
// QoS SERVICE TYPES used in making reservations. 
//
typedef enum { enum_serviceOne = 1, 
                enum_Guaranteed,
                enum_serviceThree,
                enum_serviceFour,
                enum_ControlledLoad } QosServiceType;

////////////////////////////////////////////////////////////////////////
// RSVP_FLOWSPEC specifies the desired Quality of Service for data flow
// to a particular Service. 
//
typedef struct _RSVP_FLOWSPEC
{
    QosServiceType        type;         // QoS Service type
    LONG                  length;       // Length of entire structure
    union
    {
        CL_FLOWSPEC      clFlowspec;    // Controlled Load Flowspec
        GS_FLOWSPEC      gsFlowspec;    // Guaranteed Service Flowspec
    } flowspec;
} RSVP_FLOWSPEC, *LPRSVP_FLOWSPEC;

typedef const RSVP_FLOWSPEC *LPCRSVP_FLOWSPEC;

//macro for RSVP_FLOWSPEC length
#define RSVP_FLOWSPEC_LENGTH(FLOWSPEC_TYPE) \
                    (sizeof(QosServiceType) + sizeof(LONG) + sizeof(FLOWSPEC_TYPE))

////////////////////////////////////////////////////////////////////////
// RSVP ONE PASS WITH ADVERTISING (OPWA) RELATED DEFINTIONS
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// HOP_COUNT represents the number of nodes passed in the network while
// gathering ADSPEC data. 
// 
typedef struct _HOP_COUNT
{
    BYTE            isValid;         //Zero means value is NOT valid
    BYTE            isPresent;       //Is the value present? 
    USHORT          count;           //Number of rsvp nodes passed
} HOP_COUNT, *LPHOP_COUNT;

typedef const HOP_COUNT *LPCHOP_COUNT;

////////////////////////////////////////////////////////////////////////
// PATH_BANDWIDTH represents the bandwidth estimate for the network path
//
typedef struct _PATH_BANDWIDTH
{
    BYTE            isValid;          //Zero means value is NOT valid
    BYTE            isPresent;        //Is the value present? 
    FLOAT           bandwidth;        //Bandwidth estimate in bytes
} PATH_BANDWIDTH, *LPPATH_BANDWIDTH;

typedef const PATH_BANDWIDTH *LPCPATH_BANDWIDTH;

////////////////////////////////////////////////////////////////////////
// PATH_LATENCY represents the minimum path latency
// for a network path.
//
typedef struct _PATH_LATENCY
{
    BYTE            isValid;          //Zero means value is NOT valid
    BYTE            isPresent;        //Is the value present? 
    LONG            latency;          //Min Path Latency in microseconds
} PATH_LATENCY, *LPPATH_LATENCY;

typedef const PATH_LATENCY *LPCPATH_LATENCY;

////////////////////////////////////////////////////////////////////////
// PATH_MTU represents the MTU (maximum transmission unit) supported
// by a QoS service along a network path. 
//
typedef struct _PATH_MTU
{
    BYTE            isValid;         //Zero means value is NOT valid
    BYTE            isPresent;       //Is the value present? 
    USHORT          MTU;             //Composed path MTU in bytes
} PATH_MTU, *LPPATH_MTU;

typedef const PATH_MTU *LPCPATH_MTU;

////////////////////////////////////////////////////////////////////////
// DEF_ADSPEC_PARAMS represents all defined characterization 
// parameters for an ADSPEC type.  
// 
typedef struct _DEF_ADSPEC_PARAMS
{
    BYTE            globalBreak;        //Global break bit 
                                        // ...(break in QoS control)
    HOP_COUNT       hopCount;           //Hop Count
    PATH_BANDWIDTH  pathBW;             //Estimated path bandwidth
    PATH_LATENCY    pathLatency;        //Minimum path latency
    PATH_MTU        pathMTU;            //Composed MTU value 
} DEF_ADSPEC_PARAMS, *LPDEF_ADSPEC_PARAMS;

typedef const DEF_ADSPEC_PARAMS *LPCDEF_ADSPEC_PARAMS;

////////////////////////////////////////////////////////////////////////
// GS_ADSPEC_DATA represents the adspec type for Guaranteed 
// QoS Service.
//
typedef struct _GS_ADSPEC_DATA
{
    LONG    length;                           //Total length of structure
                                              //...including params if present 
    LONG    totalC;                           //Composed Ctot value
    LONG    totalD;                           //Composed Dtot value
    LONG    sumC;                             //Composed Csum value
    LONG    sumD;                             //Composed Dsum value
    LPDEF_ADSPEC_PARAMS lpOverrideParams;     //Param values to override
                                              //... NULL if omitted
} GS_ADSPEC_DATA, *LPGS_ADSPEC_DATA;

typedef const GS_ADSPEC_DATA *LPCGS_ADSPEC_DATA;

////////////////////////////////////////////////////////////////////////
// CL_ADSPEC_DATA represents the adspec type for Controlled Load Service
// The characteristics of this type is in the Integrated Services draft. 
//
typedef struct _CL_ADSPEC_DATA
{
   LONG     length;                           //Total length of structure
                                              //... including params if present
   LPDEF_ADSPEC_PARAMS lpOverrideParams;      //Param values to override
                                              //... NULL if omitted
} CL_ADSPEC_DATA, *LPCL_ADSPEC_DATA;

typedef const CL_ADSPEC_DATA *LPCCL_ADSPEC_DATA;

////////////////////////////////////////////////////////////////////////
// RSVP ADSPEC Flags used by sender to allow service requests 
// on data flow. These flags may be ORed together to allow for multiple
// service requests. ie. to allow both Guaranteed and Control Load,
// use (ADSPEC_GS_OKAY|ADSPEC_CL_OKAY).
//
typedef LONG RSVP_ADSPEC_FLAG;

//Services or Level of Guarantees that may be requested for a Sender.
//These may be ORed together
//
#define ADSPEC_GS_OKAY  ((RSVP_ADSPEC_FLAG)0x01)   //Guaranteed Service 
#define ADSPEC_CL_OKAY  ((RSVP_ADSPEC_FLAG)0x02)   //Controlled Load 

////////////////////////////////////////////////////////////////////////
// SENDER_ADSPEC supplied by sender for RSVP One Pass With Advertising
// (OPWA).A sender specifies whether it is 'OK' to request a particular
// service on it's data flow. 
//
typedef struct _SENDER_ADSPEC
{
    RSVP_ADSPEC_FLAG    okayFlag;   //service(s) that may be requested
                                    //...ie (ADSPEC_GS_OKAY|ADSPEC_CL_OKAY)
} SENDER_ADSPEC, *LPSENDER_ADSPEC;

typedef const SENDER_ADSPEC *LPCSENDER_ADSPEC;

////////////////////////////////////////////////////////////////////////
// RSVP_ADSPEC is used for RSVP One Pass With Advertising (OPWA).
//
typedef struct _RSVP_ADSPEC
{
    LONG                length;       //total length of the structure
                                      //...including adspecs if present  
    DEF_ADSPEC_PARAMS   genParams;    //Default general parameters
    LPGS_ADSPEC_DATA    lpGSAdSpec;   //GS AdSpec data; NULL if omitted
    LPCL_ADSPEC_DATA    lpCLAdSpec;   //CL AdSpec data; NULL if omitted   
} RSVP_ADSPEC, *LPRSVP_ADSPEC;

typedef const RSVP_ADSPEC *LPCRSVP_ADSPEC;

////////////////////////////////////////////////////////////////////////
// ENDPOINT_ADDR Identifies the IP Address and Port of an Endpoint in
// an RSVP session. This provides support for both IPv4 AND IPv6.
// At compile time or in an application header define 'RSVP_IPV6'
// to enable support for IPV6 addressing. If this definition is
// not present then the Winsock2 sockaddr_in definition is used.
// The'sin_family' structure must be set to 'AF_INET' for IPv4 and
// 'AF_INET6' for IPv6 as defined in Winsock2.h.
//
#ifdef RSVP_IPV6  //Enable IPV6 support

    typedef struct _IN_ADDR_IPV6
    {
        BYTE  s_addrv6[16];               //IPV6 address
    } IN_ADDR_IPV6, *LPIN_ADDR_IPV6;

    typedef const IN_ADDR_IPV6  *LPCIN_ADDR_IPV6;

    typedef struct _ENDPOINT_ADDR
    {
        SHORT           sin_family;     //AF_INET6
        USHORT          sin_port;       //Transport level port number
        ULONG           sin_flowinfo;   //IPV6 related flow information
        IN_ADDR_IPV6    sin_addr;       //IPV6 address
    } ENDPOINT_ADDR, *LPENDPOINT_ADDR;

#else  //use IPV4 structure defined in Winsock2

    typedef SOCKADDR_IN ENDPOINT_ADDR, *LPENDPOINT_ADDR;

#endif //RSVP_IPV6

typedef const ENDPOINT_ADDR *LPCENDPOINT_ADDR;

////////////////////////////////////////////////////////////////////////
// RSVP_SESSION identifies a session.  A session is a particular
// destination (either unicast address or multicast group) to which one
// or more senders may direct a data stream.  A session is identified by
// a unique combination of IP address, protocol ID and optional port
// number.
//
typedef struct _RSVP_SESSION
{
    ENDPOINT_ADDR       addr;             //Destination address and port
    BYTE                protId;           //Session protocol ID
} RSVP_SESSION, *LPRSVP_SESSION;

typedef const RSVP_SESSION *LPCRSVP_SESSION;

////////////////////////////////////////////////////////////////////////
// RSVP_SENDER_TEMPLATE identifies a unique sender within a session.
//
typedef ENDPOINT_ADDR  RSVP_SENDER_TEMPLATE, *LPRSVP_SENDER_TEMPLATE; 
typedef const RSVP_SENDER_TEMPLATE *LPCRSVP_SENDER_TEMPLATE;

////////////////////////////////////////////////////////////////////////
// RSVP_FILTERSPEC selects a particular sender within a session.  It is
// identical to RSVP_SENDER_TEMPLATE.
//
typedef RSVP_SENDER_TEMPLATE RSVP_FILTERSPEC, *LPRSVP_FILTERSPEC;
typedef const RSVP_FILTERSPEC *LPCRSVP_FILTERSPEC;

/*
////////////////////////////////////////////////////////////////////////
// RSVP_POLICY contains undefined policy data.  RSVP transports this
// data on behalf of the Policy Control component.
//
typedef struct _RSVP_POLICY
{
   LPVOID       lpType;              // Pointer to structure: TBD
} RSVP_POLICY, *LPRSVP_POLICY;

typedef const RSVP_POLICY *LPCRSVP_POLICY;

// RSVP_USERNAME_POLICY
typedef struct _USERNAME_POLICY
{
    const char *  lpUsername;     // points to a null terminated string
} USERNAME_POLICY, *LPUSERNAME_POLICY;

*/

////////////////////////////////////////////////////////////////////////
// RSVP_FLOWDESC specifies one or more data flows.
//
typedef struct _RSVP_FLOWDESC
{
    RSVP_FLOWSPEC         flow;               // Flowspec
    LPVOID                lpPolicy;           // Optional policy data
                                              // ...NULL if omitted
    ULONG                 numFilts;           // Number of filterspecs
                                              //... zero or more
    LPRSVP_FILTERSPEC     filtList;           // Filterspec list
} RSVP_FLOWDESC, *LPRSVP_FLOWDESC;

typedef const RSVP_FLOWDESC *LPCRSVP_FLOWDESC;

////////////////////////////////////////////////////////////////////////
// RSVP_INTEGRITY contains authentication data.  A client which
// supplies integrity data must comply with Internet Draft "RSVP
// Cryptographic Authentication", draft-ietf-rsvp-md5-01.
//
typedef struct _RSVP_INTEGRITY
{
    ULONG              keyId;           // Key identifier
    ULONG              seqNo;           // Sequence number
    ENDPOINT_ADDR      addrSource;      // Sender's IP address
    ULONG              octetId;         // octetId    
    LPBYTE             lpMsgDigest;     // Message Digest 
                                        // ...for Authentication
} RSVP_INTEGRITY, *LPRSVP_INTEGRITY;

// OCTET_IDs
// Number of octets used to define Digest: See specification.
// This must be a multiple of four. Four or 16 supported currently

//4 octects used for digest
#define OCTET_4            ((OCTET_ID)0x04) 

//16 octets used as defined in md5
#define OCTET_MD5          ((OCTET_ID)0x10)   

typedef const RSVP_INTEGRITY *LPCRSVP_INTEGRITY;

////////////////////////////////////////////////////////////////////////
// RSVP PROGRAMMING INTERFACE RELATED DEFINITIONS : IOCTLs
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// REGISTER_DATA is used to issue the SIO_RSVP_REGISTER command.  This
// command establishes an association between a RSVP session and a
// particular socket, and informs RSVP of the caller's interest in RSVP
// events.
//
typedef struct _REGISTER_DATA
{
    RSVP_SESSION    sess;            // Session of interest
    ULONG           regOption;       // Various options are given below
} REGISTER_DATA, *LPREGISTER_DATA;

typedef const REGISTER_DATA *LPCREGISTER_DATA;

// REGISTER_DATA option values: 
//A host may only register for a session as a sender, receiver or both.
//
#define RSVP_SENDER         0x01
#define RSVP_RECEIVER       0x02
#define RSVP_SENRECV        0x03


////////////////////////////////////////////////////////////////////////
// SENDER_DATA is used to issue the SIO_RSVP_SENDER command.  This
// command informs interested receivers of the source and traffic
// characteristics of a data flow.
typedef struct _SENDER_DATA
{
    RSVP_SESSION            sess;           // Session of interest
    ENDPOINT_ADDR           localAddr;      // local address and port
    RSVP_SENDER_TSPEC       tspec;          // Traffic specification
    LPVOID                  lpPolicy;       // Optional policy data; 
                                            // ...NULL if omitted
    UINT                    dataTTL;        // Optional TTL; Zero if omitted
} SENDER_DATA, *LPSENDER_DATA;

typedef const SENDER_DATA *LPCSENDER_DATA;


////////////////////////////////////////////////////////////////////////
// Resv Confirm flags 
#define RESV_CONFIRM_NOT_REQUIRED   0
#define RESV_CONFIRM_REQUIRED       1

////////////////////////////////////////////////////////////////////////
// RESERVE_DATA is used to issue the SIO_RSVP_RESERVE command.  This
// command either establishes a new reservation or modifies an existing
// one.
typedef struct _RESERVE_DATA
{
    RSVP_SESSION        sess;           // Session of interest
    LPVOID              lpPolicy;       // Optional policy data
                                        // ...NULL if omitted
    ULONG               style;          // Style Of reservation (defined below)
    BYTE                confmFlag;      // Non Zero if RESV_CONFM is needed
    ENDPOINT_ADDR       hostAddr;       // Optional; local interface handle
    ULONG               numFds;         // Number Of flowdescriptors; min zero
    LPRSVP_FLOWDESC     fdList;         // Flowdescriptor list
} RESERVE_DATA, *LPRESERVE_DATA;

typedef const RESERVE_DATA *LPCRESERVE_DATA;

// Supported Reservation style IDs
#define RSVP_STYLE_FIXED_FILTER       0x0000000A
#define RSVP_STYLE_SHARED_EXPLICIT    0x00000012
#define RSVP_STYLE_WILDCARD_FILTER    0x00000011

////////////////////////////////////////////////////////////////////////
// REL_SENDER_DATA is used to issue the SIO_RSVP_REL_SENDER command.
// This command removes all reservations established against the
// specified data flow.
typedef struct _REL_SENDER_DATA
{
    RSVP_SESSION            sess;           // Session of interest
    RSVP_SENDER_TEMPLATE    sender;         // Sender template
} REL_SENDER_DATA, *LPREL_SENDER_DATA;

typedef const REL_SENDER_DATA *LPCREL_SENDER_DATA;


////////////////////////////////////////////////////////////////////////
// REL_RESERVE_DATA is used to issue the SIO_RSVP_REL_RESERVE WSAIoctl()
// command.  This command releases an existing reservation.
typedef struct _REL_RESERVE_DATA
{
    RSVP_SESSION        sess;           // Session of interest
    ENDPOINT_ADDR       hostAddr;       // Optional; address on which
                                        // reservation was made
    ULONG               style;          // Style reserved
    ULONG               numFilts;       // Number of filterspecs
    LPRSVP_FILTERSPEC   filtList;       // Filterspec list
} REL_RESERVE_DATA, *LPREL_RESERVE_DATA;

typedef const REL_RESERVE_DATA *LPCREL_RESERVE_DATA;


////////////////////////////////////////////////////////////////////////
// QUERY_QOS_DATA is used to issue the SIO_RSVP_QUERY_QOS command.
// This command interogates the status of a particular reservation.
typedef struct _QUERY_QOS_DATA
{
    RSVP_SESSION        sess;           // Session of interest
    ULONG               numFilts;       // Number of filters; if Zero see below
    LPRSVP_FILTERSPEC   filtList;       // Optional.. If null, all
                                        // filterspecs for the session
                                        // are returned.
} QUERY_QOS_DATA, *LPQUERY_QOS_DATA;

typedef const QUERY_QOS_DATA *LPCQUERY_QOS_DATA;


////////////////////////////////////////////////////////////////////////
// RSVP_ERROR returns a RSVP error code for a particular session.
typedef struct _RSVP_ERROR
{
    RSVP_SESSION    sess;               // Session of interest
    LONG            errCode;            // RSVP error code
} RSVP_ERROR, *LPRSVP_ERROR;

typedef const RSVP_ERROR *LPCRSVP_ERROR;

////////////////////////////////////////////////////////////////////////
// RSVP_SNDDESC describes a particular sender.
typedef struct _RSVP_SNDDESC
{
    RSVP_SESSION                 sess;      // Session identification  
    RSVP_SENDER_TEMPLATE         sender;    // Sender template
    RSVP_SENDER_TSPEC            tspec;     // Sender traffic specification
    RSVP_ADSPEC                  adspec;    // Sender adspec
    LPVOID                       lpPolicy;  // Optional policy data
} RSVP_SNDDESC, *LPRSVP_SNDDESC;

typedef const RSVP_SNDDESC *LPCRSVP_SNDDESC;


////////////////////////////////////////////////////////////////////////
// RSVP_PATHERR indicates the reason that a Path request failed or was
// rejected. All error codes are RSVP protocol specific.
typedef struct _RSVP_PATHERR
{
    BYTE                    eCode;       // Protocol specific error code
    LONG                    eValue;      // Protocol specific error value
    ENDPOINT_ADDR           eNodeAddr;   // Source node of error
    RSVP_SENDER_TEMPLATE    eSender;     // Sender template
    LPVOID                  lpPolicy;    // Optional policy data
} RSVP_PATHERR, *LPRSVP_PATHERR;

typedef const RSVP_PATHERR *LPCRSVP_PATHERR;


////////////////////////////////////////////////////////////////////////
// RSVP_RESVERR indicates the reason that a reservation failed or was
// terminated. All errors are RSVP protocol specific.
typedef struct _RSVP_RESVERR
{
    BYTE                eCode;        // Protocol specific error code
    LONG                eValue;       // Protocol specific error value
    ENDPOINT_ADDR       eNodeAddr;    // Source node of error
    ULONG               numFds;       // Number of flowdescriptors
    LPRSVP_FLOWDESC     fdList;       // Flowdescriptors to which
                                      // ...error pertains
} RSVP_RESVERR, *LPRSVP_RESVERR;

typedef const RSVP_RESVERR *LPCRSVP_RESVERR;

////////////////////////////////////////////////////////////////////////
// RSVP_RESVMSG Structure used for storing reserve data
typedef struct _RSVP_RESVMSG
{
    ENDPOINT_ADDR       nHop;       //Source of resv mesg (host,router)
    LPVOID              lpPolicy;   //Optional policy data
    ULONG               style;      //Style of reservation
    ULONG               numFds;     //Number of FlowDescriptors
    LPRSVP_FLOWDESC     fdList;     //Flowdescriptor list
} RSVP_RESVMSG, *LPRSVP_RESVMSG;

typedef const RSVP_RESVMSG *LPCRSVP_RESVMSG;

////////////////////////////////////////////////////////////////////////
// RSVP_RESVCONF Structure used for storing reserve data
typedef struct _RSVP_RESVCONF
{
    ENDPOINT_ADDR       source;     //Source of resv conf mesg
    ULONG               style;      //Style of reservation
    ULONG               numFds;     //Number of FlowDescriptors
    LPRSVP_FLOWDESC     fdList;     //Flowdescriptor list
} RSVP_RESVCONF, *LPRSVP_RESVCONF;

typedef const RSVP_RESVCONF *LPCRSVP_RESVCONF;


////////////////////////////////////////////////////////////////////////
// RSVP information messages.
typedef LONG                RSVPINFO;

#define INFO_PATH           ((RSVPINFO) 101)
#define INFO_PATH_TEAR      ((RSVPINFO) 102)
#define INFO_PATH_ERR       ((RSVPINFO) 103)
#define INFO_PATH_TIMEOUT   ((RSVPINFO) 104)
#define INFO_RESV           ((RSVPINFO) 105)
#define INFO_RESV_TEAR      ((RSVPINFO) 106)
#define INFO_RESV_ERR       ((RSVPINFO) 107)
#define INFO_RESV_CHANGE    ((RSVPINFO) 108)
#define INFO_RESV_TIMEOUT   ((RSVPINFO) 109)
#define INFO_RESV_CONFIRM   ((RSVPINFO) 110)
#define INFO_QUERY_QOS      ((RSVPINFO) 111)
#define INFO_ERROR          ((RSVPINFO) 112)

////////////////////////////////////////////////////////////////////////
// RSVP Status indications from GQOS to Applications
// Based on Winsock2 WSABASEERR ... 

#define INFO_WSAE_RECEIVERS           (WSABASEERR+350)
#define INFO_WSAE_SENDERS             (WSABASEERR+351)
#define INFO_WSAE_NOSENDERS           (WSABASEERR+352)
#define INFO_WSAE_NORECEIVERS         (WSABASEERR+353)
#define INFO_WSAE_REQCONFIRMED        (WSABASEERR+354)
#define INFO_WSAE_QOSCHANGE           (WSABASERRR+354)
#define INFO_WSAE_ADMFAILURE          (WSABASEERR+356)
#define INFO_WSAE_POLFAILURE          (WSABASEERR+357)
#define INFO_WSAE_TCERROR             (WSABASEERR+358)
#define INFO_WSAE_GENERROR            (WSABASEERR+359)

////////////////////////////////////////////////////////////////////////
// RSVP Constants for QOS structure

#define RSVP_QOSUNSPECIFIED           ((UINT32)-1) 


////////////////////////////////////////////////////////////////////////
// RSVP_RETURN contains an RSVP information message.  infoType indicates
// which ret union member should be used.
typedef struct _RSVP_RETURN
{
    RSVP_SESSION            sess;           // Session of interest
    LONG                    infoLength;     // Length of returned buffer
    RSVPINFO                infoType;       // Returned information type
    union
    {
        LPRSVP_SNDDESC      sndDesc;        // Path or Path Tear
        LPRSVP_PATHERR      pathErr;        // Path error
        LPRSVP_RESVERR      resvErr;        // Reserve error
        LPRSVP_FLOWDESC     resvChg;        // Reservation change
        LPRESERVE_DATA      queryQoS;       // QueryQoS response
        LPRSVP_RESVMSG      resvMsg;        // Reserve Message
        LPRSVP_RESVCONF     resvConf;       // Reserve Confirm Message
        LPRSVP_ERROR        rsvpErr;        // Other errors
    } ret;
} RSVP_RETURN, *LPRSVP_RETURN;

typedef const RSVP_RETURN *LPCRSVP_RETURN;

#define RSVP_RETURN_SIZE \
    max(sizeof(RSVP_RETURN), sizeof(RSVP_ERROR))

////////////////////////////////////////////////////////////////////////
// RSVP error codes
////////////////////////////////////////////////////////////////////////
typedef BYTE                RSVPERROR;

// Message interpreted succesfully
#define RS_OK               ((RSVPERROR) 1)

// Admission control failure: delay request cannot be met
#define RS_ADM_DELAY        ((RSVPERROR) 2)

// Admission control failure: band width request cannot be met
#define RS_ADM_BW           ((RSVPERROR) 3)

// Admission control failure: service request conflicts
#define RS_ADM_SERC         ((RSVPERROR) 4)

// Admission control failure: request is unsupported
#define RS_ADM_SERU         ((RSVPERROR) 5)

// Admission control failure: request is unreasonable
#define RS_ADM_UNR          ((RSVPERROR) 6)

// Policy control failure: the required credentials are not present
#define RS_POLICY_CRED      ((RSVPERROR) 7)

// Policy control failure: request exceeds policy restrictions
#define RS_POLICY_LAR       ((RSVPERROR) 8)

// Policy control failure: reservation request exceeds policy quota
#define RS_POLICY_QUOTA     ((RSVPERROR) 9)

// Policy control failure: reservation has been preempted due to a policy
#define RS_POLICY_PREMPT    ((RSVPERROR) 10)

// A "premature" reservation (one made before a sender is known) has
// timed out awaiting a sender.
#define RS_PATH             ((RSVPERROR) 11)

// A path message contains an ambiguous path.  The sender port was
// specified as both zero and non-zero for the same session.
#define RS_AMB_PATH         ((RSVPERROR) 12)

// A reservation contains an ambiguous filterspec.  A filterspec matches
// more than one sender in a reservation style that requires a unique
// match.
#define RS_AMB_FILT          ((RSVPERROR) 14)

// A reservation a style that conflicts with an
// existing reservation.
#define RS_CONF_STYLE        ((RSVPERROR) 15)

// Conflicting destination ports
#define RS_CONF_DSTP         ((RSVPERROR) 16)

// Confliction ports in the sender template
#define RS_CONF_SRCP         ((RSVPERROR) 17)

// Missing SESSION object
#define RS_MISS_SESS         ((RSVPERROR) 18)

// For a multihomed host, the hostaddr field was missing
#define RS_MISS_STMPL        ((RSVPERROR) 19)

// Missing sender Tspec
#define RS_MISS_STSPEC       ((RSVPERROR) 20)

// Missing reservation style
#define RS_MISS_STYLE       ((RSVPERROR) 21)

// Missing FLOWSPEC object
#define RS_MISS_FLOW        ((RSVPERROR) 22)

// Missing filterspec
#define RS_MISS_FILT        ((RSVPERROR) 23)

// Invalid session object
#define RS_INV_SESS         ((RSVPERROR) 24)

// Invalid reservation style
#define RS_INV_STYLE        ((RSVPERROR) 25)

// Invalid flowspec
#define RS_INV_FLOWSPEC     ((RSVPERROR) 26)

// Invalid filterspec
#define RS_INV_FILT         ((RSVPERROR) 27)

// Invalid host address
#define RS_INV_HOSTADDR     ((RSVPERROR) 28)

// Invalid sender Tspec
#define RS_INV_STSPEC       ((RSVPERROR) 29)

// Invalid ADSPEC object
#define RS_INV_ADSPEC       ((RSVPERROR) 30)

// Invalid policy data
#define RS_INV_POLICY       ((RSVPERROR) 31)

// Error sent by the traffic control module
#define RS_ERR_TC           ((RSVPERROR) 32)

// System error, not concerning RSVP state or protocol
#define RS_ERR_SYS          ((RSVPERROR) 33)

// Reservation made is greater than requested due to merging.
#define RS_RESV_MERGD       ((RSVPERROR) 34)

// Not registered appropriately sender or receiver
#define RS_NOT_REG          ((RSVPERROR) 35)

// No information is available to be returned in a retrieve call
#define RS_NO_INFO          ((RSVPERROR) 36)

// An unspecified parameter is invalid
#define RS_INVAL            ((RSVPERROR) 37)

// Session object incorrect
#define RS_SESS             ((RSVPERROR) 38)

// An unsupported protocol was specified
#define RS_NOSP_PROT        ((RSVPERROR) 39)

// No buffers available
#define RS_NOBUFS           ((RSVPERROR) 40)
                      
// The underlying network has failed
#define  RS_FAIL_NETWORK    ((RSVPERROR) 41)

// Unrecovarable error has occurred
#define  RS_ERR_CLOSEWAIT   ((RSVPERROR) 42)

// Specified reservation style not supported
#define RS_NOSP_STYLE       ((RSVPERROR) 43)

// Specified flowspec type not supported
#define RS_NOSP_FLOWSPEC    ((RSVPERROR) 44)

// Invalid TSpec, perhaps due to zero bucket rate or depth
#define RS_INV_TSPEC        ((RSVPERROR) 45)

// Invalid RSpec, perhaps due to zero delay
#define RS_INV_RSPEC        ((RSVPERROR) 46)

// Invalid sender template
#define RS_INV_STMPL        ((RSVPERROR) 47)

// Option not supported
#define RS_NOSP_OPT         ((RSVPERROR) 48)

// The number of flowdescs specified exceeded MAX_FLOWDESC
#define RS_MAX_FLOWDESC     ((RSVPERROR) 49)

// The number of filterspecs specified exceeded MAX_FILTERS
#define RS_MAX_FILTERS      ((RSVPERROR) 50)

// The caller's buffer is too small
#define RS_INAD_BUFFER      ((RSVPERROR) 51)

// To indicate no support for ip address type
#define RS_NOSP_INFAMILY    ((RSVPERROR) 52)

#endif // (_W2RSVP_HDR)
